"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_js_1 = require("./utils.js");
/**
 * Inject the CSS compiled with JS.
 *
 * @return {Plugin}
 */
function cssInjectedByJsPlugin({ injectCode, injectCodeFunction, jsAssetsFilterFunction, preRenderCSSCode, relativeCSSInjection, styleId, suppressUnusedCssWarning, topExecutionPriority, useStrictCSP, } = {}) {
    // Globally so we can add it to legacy and non-legacy bundle.
    let globalCssToInject = '';
    let config;
    const topExecutionPriorityFlag = typeof topExecutionPriority == 'boolean' ? topExecutionPriority : true;
    return {
        apply: 'build',
        enforce: 'post',
        name: 'vite-plugin-css-injected-by-js',
        configResolved(_config) {
            config = _config;
        },
        generateBundle(opts, bundle) {
            return __awaiter(this, void 0, void 0, function* () {
                if (config.build.ssr) {
                    return;
                }
                const htmlFiles = Object.keys(bundle).filter((i) => i.endsWith('.html'));
                const cssAssets = Object.keys(bundle).filter((i) => bundle[i].type == 'asset' && bundle[i].fileName.endsWith('.css'));
                for (const name of htmlFiles) {
                    const htmlChunk = bundle[name];
                    let replacedHtml = htmlChunk.source instanceof Uint8Array
                        ? new TextDecoder().decode(htmlChunk.source)
                        : `${htmlChunk.source}`;
                    cssAssets.forEach(function replaceLinkedStylesheetsHtml(cssName) {
                        replacedHtml = (0, utils_js_1.removeLinkStyleSheets)(replacedHtml, cssName);
                        htmlChunk.source = replacedHtml;
                    });
                }
                const buildCssCode = (cssToInject) => (0, utils_js_1.buildCSSInjectionCode)({
                    cssToInject: typeof preRenderCSSCode == 'function' ? preRenderCSSCode(cssToInject) : cssToInject,
                    styleId,
                    injectCode,
                    injectCodeFunction,
                    useStrictCSP,
                    buildOptions: config.build
                });
                if (relativeCSSInjection) {
                    const assetsWithCss = (0, utils_js_1.buildJsCssMap)(bundle, jsAssetsFilterFunction);
                    yield (0, utils_js_1.relativeCssInjection)(bundle, assetsWithCss, buildCssCode);
                    if (!suppressUnusedCssWarning) {
                        // With all used CSS assets now being removed from the bundle, navigate any that have not been linked and output
                        const unusedCssAssets = cssAssets.filter((cssAsset) => !!bundle[cssAsset]).join(',');
                        unusedCssAssets.length > 0 &&
                            (0, utils_js_1.warnLog)(`[vite-plugin-css-injected-by-js] Some CSS assets were not included in any known JS: ${unusedCssAssets}`);
                    }
                    return;
                }
                // Non-relative / Global CSS injection path
                const jsAssetTargets = (0, utils_js_1.getJsAssetTargets)(bundle, jsAssetsFilterFunction);
                if (jsAssetTargets.length == 0) {
                    throw new Error('Unable to locate the JavaScript asset for adding the CSS injection code. It is recommended to review your configurations.');
                }
                process.env.VITE_CSS_INJECTED_BY_JS_DEBUG &&
                    (0, utils_js_1.debugLog)(`[vite-plugin-css-injected-by-js] Global CSS Assets: [${cssAssets.join(',')}]`);
                const allCssCode = (0, utils_js_1.concatCssAndDeleteFromBundle)(bundle, cssAssets);
                if (allCssCode.length > 0) {
                    globalCssToInject = allCssCode;
                }
                const globalCssInjectionCode = yield buildCssCode(globalCssToInject);
                for (const jsAsset of jsAssetTargets) {
                    process.env.VITE_CSS_INJECTED_BY_JS_DEBUG &&
                        (0, utils_js_1.debugLog)(`[vite-plugin-css-injected-by-js] Global CSS inject: ${jsAsset.fileName}`);
                    const appCode = jsAsset.code;
                    jsAsset.code = topExecutionPriorityFlag ? '' : appCode;
                    jsAsset.code += globalCssInjectionCode ? globalCssInjectionCode.code : '';
                    jsAsset.code += !topExecutionPriorityFlag ? '' : appCode;
                }
            });
        },
    };
}
exports.default = cssInjectedByJsPlugin;
